@@ -13,7 +13,7 @@ int main() {
       "luck. Unfortunately, Dane's luck wasn't as good as his old man's.");
    const std::vector<uint8_t> key = Botan::hex_decode("2B7E151628AED2A6ABF7158809CF4F3C");
 
-   auto enc = Botan::Cipher_Mode::create("AES-128/CBC/PKCS7", Botan::Cipher_Dir::Encryption);
+   const auto enc = Botan::Cipher_Mode::create_or_throw("AES-128/CBC/PKCS7", Botan::Cipher_Dir::Encryption);
    enc->set_key(key);
 
    // generate fresh nonce (IV)
@@ -25,6 +25,6 @@ int main() {
    enc->start(iv);
    enc->finish(pt);
 
-   std::cout << enc->name() << " with iv " << Botan::hex_encode(iv) << " " << Botan::hex_encode(pt) << "\n";
+   std::cout << enc->name() << " with iv " << Botan::hex_encode(iv) << " " << Botan::hex_encode(pt) << '\n';
    return 0;
 }
