@@ -46,6 +46,10 @@
    #include <botan/ed25519.h>
 #endif
 
+#if defined(BOTAN_HAS_ED448)
+   #include <botan/ed448.h>
+#endif
+
 #if defined(BOTAN_HAS_GOST_34_10_2001)
    #include <botan/gost_3410.h>
 #endif
@@ -62,6 +66,10 @@
    #include <botan/curve25519.h>
 #endif
 
+#if defined(BOTAN_HAS_X448)
+   #include <botan/x448.h>
+#endif
+
 #if defined(BOTAN_HAS_MCELIECE)
    #include <botan/mceliece.h>
 #endif
@@ -110,6 +118,12 @@ std::unique_ptr<Public_Key> load_public_key(const AlgorithmIdentifier& alg_id,
    }
 #endif
 
+#if defined(BOTAN_HAS_X448)
+   if(alg_name == "X448") {
+      return std::make_unique<X448_PublicKey>(alg_id, key_bits);
+   }
+#endif
+
 #if defined(BOTAN_HAS_MCELIECE)
    if(alg_name == "McEliece") {
       return std::make_unique<McEliece_PublicKey>(key_bits);
@@ -176,6 +190,12 @@ std::unique_ptr<Public_Key> load_public_key(const AlgorithmIdentifier& alg_id,
    }
 #endif
 
+#if defined(BOTAN_HAS_ED448)
+   if(alg_name == "Ed448") {
+      return std::make_unique<Ed448_PublicKey>(alg_id, key_bits);
+   }
+#endif
+
 #if defined(BOTAN_HAS_GOST_34_10_2001)
    if(alg_name == "GOST-34.10" || alg_name == "GOST-34.10-2012-256" || alg_name == "GOST-34.10-2012-512") {
       return std::make_unique<GOST_3410_PublicKey>(alg_id, key_bits);
@@ -227,6 +247,12 @@ std::unique_ptr<Private_Key> load_private_key(const AlgorithmIdentifier& alg_id,
    }
 #endif
 
+#if defined(BOTAN_HAS_X448)
+   if(alg_name == "X448") {
+      return std::make_unique<X448_PrivateKey>(alg_id, key_bits);
+   }
+#endif
+
 #if defined(BOTAN_HAS_ECDSA)
    if(alg_name == "ECDSA") {
       return std::make_unique<ECDSA_PrivateKey>(alg_id, key_bits);
@@ -287,6 +313,12 @@ std::unique_ptr<Private_Key> load_private_key(const AlgorithmIdentifier& alg_id,
    }
 #endif
 
+#if defined(BOTAN_HAS_ED448)
+   if(alg_name == "Ed448") {
+      return std::make_unique<Ed448_PrivateKey>(alg_id, key_bits);
+   }
+#endif
+
 #if defined(BOTAN_HAS_GOST_34_10_2001)
    if(alg_name == "GOST-34.10" || alg_name == "GOST-34.10-2012-256" || alg_name == "GOST-34.10-2012-512") {
       return std::make_unique<GOST_3410_PrivateKey>(alg_id, key_bits);
@@ -385,6 +417,12 @@ std::unique_ptr<Private_Key> create_private_key(std::string_view alg_name,
    }
 #endif
 
+#if defined(BOTAN_HAS_X448)
+   if(alg_name == "X448") {
+      return std::make_unique<X448_PrivateKey>(rng);
+   }
+#endif
+
 #if defined(BOTAN_HAS_RSA)
    if(alg_name == "RSA") {
       const size_t modulus_bits = params.empty() ? 3072 : to_u32bit(params);
@@ -474,6 +512,12 @@ std::unique_ptr<Private_Key> create_private_key(std::string_view alg_name,
    }
 #endif
 
+#if defined(BOTAN_HAS_ED448)
+   if(alg_name == "Ed448") {
+      return std::make_unique<Ed448_PrivateKey>(rng);
+   }
+#endif
+
    // ECC crypto
 #if defined(BOTAN_HAS_ECC_PUBLIC_KEY_CRYPTO)
 
