@@ -1,5 +1,6 @@
 /*
 * (C) 2009,2015,2016 Jack Lloyd
+* (C) 2024           Fabian Albert, René Meusel -  Rohde & Schwarz Cybersecurity
 *
 * Botan is released under the Simplified BSD License (see license.txt)
 */
@@ -80,36 +81,36 @@ class BigInt_Unit_Tests final : public Test {
       static Test::Result test_random_prime() {
          Test::Result result("BigInt prime generation");
 
-         result.test_throws("Invalid bit size", "random_prime: Can't make a prime of 0 bits", []() {
-            Botan::random_prime(Test::rng(), 0);
-         });
-         result.test_throws("Invalid bit size", "random_prime: Can't make a prime of 1 bits", []() {
-            Botan::random_prime(Test::rng(), 1);
-         });
-         result.test_throws("Invalid arg", "random_prime Invalid value for equiv/modulo", []() {
-            Botan::random_prime(Test::rng(), 2, 1, 0, 2);
+         auto rng = Test::new_rng("random_prime");
+
+         result.test_throws(
+            "Invalid bit size", "random_prime: Can't make a prime of 0 bits", [&]() { Botan::random_prime(*rng, 0); });
+         result.test_throws(
+            "Invalid bit size", "random_prime: Can't make a prime of 1 bits", [&]() { Botan::random_prime(*rng, 1); });
+         result.test_throws("Invalid arg", "random_prime Invalid value for equiv/modulo", [&]() {
+            Botan::random_prime(*rng, 2, 1, 0, 2);
          });
 
-         BigInt p = Botan::random_prime(Test::rng(), 2);
+         BigInt p = Botan::random_prime(*rng, 2);
          result.confirm("Only two 2-bit primes", p == 2 || p == 3);
 
-         p = Botan::random_prime(Test::rng(), 3);
+         p = Botan::random_prime(*rng, 3);
          result.confirm("Only two 3-bit primes", p == 5 || p == 7);
 
-         p = Botan::random_prime(Test::rng(), 4);
+         p = Botan::random_prime(*rng, 4);
          result.confirm("Only two 4-bit primes", p == 11 || p == 13);
 
          for(size_t bits = 5; bits <= 32; ++bits) {
-            p = Botan::random_prime(Test::rng(), bits);
+            p = Botan::random_prime(*rng, bits);
             result.test_eq("Expected bit size", p.bits(), bits);
-            result.test_eq("P is prime", Botan::is_prime(p, Test::rng()), true);
+            result.test_eq("P is prime", Botan::is_prime(p, *rng), true);
          }
 
          const size_t safe_prime_bits = 65;
-         const BigInt safe_prime = Botan::random_safe_prime(Test::rng(), safe_prime_bits);
+         const BigInt safe_prime = Botan::random_safe_prime(*rng, safe_prime_bits);
          result.test_eq("Safe prime size", safe_prime.bits(), safe_prime_bits);
-         result.confirm("P is prime", Botan::is_prime(safe_prime, Test::rng()));
-         result.confirm("(P-1)/2 is prime", Botan::is_prime((safe_prime - 1) / 2, Test::rng()));
+         result.confirm("P is prime", Botan::is_prime(safe_prime, *rng));
+         result.confirm("(P-1)/2 is prime", Botan::is_prime((safe_prime - 1) / 2, *rng));
 
          return result;
       }
@@ -142,7 +143,9 @@ class BigInt_Unit_Tests final : public Test {
 
          const size_t rbits = 1024;
 
-         const Botan::BigInt r(Test::rng(), rbits);
+         auto rng = Test::new_rng("get_substring");
+
+         const Botan::BigInt r(*rng, rbits);
 
          for(size_t wlen = 1; wlen <= 32; ++wlen) {
             for(size_t offset = 0; offset != rbits + 64; ++offset) {
@@ -560,8 +563,6 @@ Test::Result test_const_time_left_shift() {
    return result;
 }
 
-BOTAN_REGISTER_TEST_FN("math", "bn_ct_lshift", test_const_time_left_shift);
-
 class BigInt_Powmod_Test final : public Text_Based_Test {
    public:
       BigInt_Powmod_Test() : Text_Based_Test("bn/powmod.vec", "Base,Exponent,Modulus,Output") {}
@@ -594,7 +595,7 @@ class BigInt_IsPrime_Test final : public Text_Based_Test {
          const bool is_prime = (header == "Prime");
 
          Test::Result result("BigInt Test " + header);
-         result.test_eq("is_prime", Botan::is_prime(value, Test::rng()), is_prime);
+         result.test_eq("is_prime", Botan::is_prime(value, this->rng()), is_prime);
 
          return result;
       }
@@ -717,7 +718,7 @@ class Lucas_Primality_Test final : public Test {
          for(uint32_t i = 3; i <= lucas_max; i += 2) {
             Botan::Modular_Reducer mod_i(i);
             const bool passes_lucas = Botan::is_lucas_probable_prime(i, mod_i);
-            const bool is_prime = Botan::is_prime(i, Test::rng());
+            const bool is_prime = Botan::is_prime(i, this->rng());
 
             const bool is_lucas_pp = (is_prime == false && passes_lucas == true);
 
@@ -758,7 +759,7 @@ class DSA_ParamGen_Test final : public Text_Based_Test {
 
          try {
             Botan::BigInt gen_P, gen_Q;
-            if(Botan::generate_dsa_primes(Test::rng(), gen_P, gen_Q, p_bits, q_bits, seed, offset)) {
+            if(Botan::generate_dsa_primes(this->rng(), gen_P, gen_Q, p_bits, q_bits, seed, offset)) {
                result.test_eq("P", gen_P, exp_P);
                result.test_eq("Q", gen_Q, exp_Q);
             } else {
@@ -772,6 +773,60 @@ class DSA_ParamGen_Test final : public Text_Based_Test {
 
 BOTAN_REGISTER_TEST("math", "dsa_param", DSA_ParamGen_Test);
 
+std::vector<Test::Result> test_bigint_serialization() {
+   auto rng = Test::new_rng("test_bigint_serialization");
+
+   return {
+      Botan_Tests::CHECK("BigInt binary serialization",
+                         [](Test::Result& res) {
+                            Botan::BigInt a(0x1234567890ABCDEF);
+                            auto enc = Botan::BigInt::encode(a);
+                            res.test_eq("BigInt::encode()", enc, Botan::hex_decode("1234567890ABCDEF"));
+
+                            auto enc_locked = Botan::BigInt::encode_locked(a);
+                            res.test_eq(
+                               "BigInt::encode_locked()", enc_locked, Botan::hex_decode_locked("1234567890ABCDEF"));
+                            std::vector<uint8_t> enc2(a.bytes());
+                            a.binary_encode(enc2.data(), enc2.size());
+                            res.test_eq("BigInt::binary_encode", enc2, Botan::hex_decode("1234567890ABCDEF"));
+                         }),
+
+      Botan_Tests::CHECK("BigInt truncated/padded binary serialization",
+                         [&](Test::Result& res) {
+                            Botan::BigInt a(0xFEDCBA9876543210);
+
+                            std::vector<uint8_t> enc1(a.bytes() - 1);
+                            a.binary_encode(enc1.data(), enc1.size());
+                            res.test_eq("BigInt::binary_encode", enc1, Botan::hex_decode("DCBA9876543210"));
+
+                            std::vector<uint8_t> enc2(a.bytes() - 3);
+                            a.binary_encode(enc2.data(), enc2.size());
+                            res.test_eq("BigInt::binary_encode", enc2, Botan::hex_decode("9876543210"));
+
+                            std::vector<uint8_t> enc3(a.bytes() + 1);
+                            a.binary_encode(enc3.data(), enc3.size());
+                            res.test_eq("BigInt::binary_encode", enc3, Botan::hex_decode("00FEDCBA9876543210"));
+
+                            // make sure that the padding is actually written
+                            std::vector<uint8_t> enc4(a.bytes() + 3);
+                            rng->randomize(enc4);
+                            a.binary_encode(enc4.data(), enc4.size());
+                            res.test_eq("BigInt::binary_encode", enc4, Botan::hex_decode("000000FEDCBA9876543210"));
+
+                            Botan::BigInt b(Botan::hex_decode("FEDCBA9876543210BAADC0FFEE"));
+
+                            std::vector<uint8_t> enc5(b.bytes() + 12);
+                            rng->randomize(enc5);
+                            b.binary_encode(enc5.data(), enc5.size());
+                            res.test_eq("BigInt::binary_encode",
+                                        enc5,
+                                        Botan::hex_decode("000000000000000000000000FEDCBA9876543210BAADC0FFEE"));
+                         }),
+   };
+}
+
+BOTAN_REGISTER_TEST_FN("math", "bignum_auxiliary", test_const_time_left_shift, test_bigint_serialization);
+
 #endif
 
 }  // namespace
